
#include "f2c.h"

/* Table of constant values */

static int c__9 = 9;
static int c__1 = 1;
static int c__4 = 4;


/*    1-D Cubic Spline Package. */
/*    Interpolates a function x(s) from discrete x(i) points. */

/*                                              Mark Drela */
/*                                              1985 */

/*    Usage: */

/* C---- fill S(i), X(i) arrays */
/*      S(i) = ... */
/*      X(i) = ... */

/* C---- or.. for a space curve X(i), Y(i), the spline parameter S(i) */
/* C-     can be computed by */
/*      CALL SCALC(X,Y,S,N) */

/* C---- calculate spline coefficients XS(i), YS(i) */
/* C-    (or can use SPLIND,SPLINA,SEGSPL,SEGSPD instead as needed) */
/*      CALL SPLINE(X,XS,S,N) */
/*      CALL SPLINE(Y,YS,S,N) */

/* C---- The above calls are done once, which then enables any number */
/* C     of calls to the spline interrogation routines.  Examples are below. */
/* C. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */


/* C---- evaluate splined x(s) and/or its derivatives */
/* C-     at any number of s points SS */
/*      XX   = SEVAL(SS,X,XS,S,N) */
/*      XXS  = DEVAL(SS,X,XS,S,N) */
/*      XXSS = D2VAL(SS,X,XS,S,N) */

/* C---- also evaluate y(s), etc */
/*      YY   = SEVAL(SS,Y,YS,S,N) */
/*      YYS  = DEVAL(SS,Y,YS,S,N) */

/* C---- evaluate curvature k(s) of x,y curve */
/*      CV = CURV(SS,X,XS,Y,YS,S,N) */

/* C---- alternative to calling SEVAL,DEVAL,D2VAL separately */
/* C-     (slightly more efficient if all three quantities are needed) */
/*      CALL SEVALL(SS,X,XS,S,N, XX,XXS,XXSS) */


int spline_(real *x, real *xs, real *s, int *n)
{
  /* System generated locals */
  int i__1;

  /* Local variables */
  static real a[5001], b[5001], c__[5001];
  static int i__;
  static real dsm, dsp;

  /* ------------------------------------------------------- */
  /*     Calculates spline coefficients for X(S).          | */
  /*     Natural end conditions are used (zero 3rd         | */
  /*      derivative over first, last intervals).          | */
  /*                                                       | */
  /*     To evaluate the spline at some value of S,        | */
  /*     use SEVAL and/or DEVAL.                           | */
  /*                                                       | */
  /*     S        independent variable array (input)       | */
  /*     X        dependent variable array   (input)       | */
  /*     XS       dX/dS array                (calculated)  | */
  /*     N        number of points           (input)       | */
  /*                                                       | */
  /* ------------------------------------------------------- */
  /* Parameter adjustments */
  --s;
  --xs;
  --x;

  /* Function Body */
  if (*n > 5001) {
    s_stop("SPLINE: array overflow, increase NMAX", (ftnlen)37);
  }

  /*<       DO 1 I=2, N-1 >*/
  i__1 = *n - 1;
  for (i__ = 2; i__ <= i__1; ++i__) {
    /*<         DSM = S(I) - S(I-1) >*/
    dsm = s[i__] - s[i__ - 1];
    /*<         DSP = S(I+1) - S(I) >*/
    dsp = s[i__ + 1] - s[i__];
    /*<         B(I) = DSP >*/
    b[i__ - 1] = dsp;
    /*<         A(I) = 2.0*(DSM+DSP) >*/
    a[i__ - 1] = (dsm + dsp) * 2.f;
    /*<         C(I) = DSM >*/
    c__[i__ - 1] = dsm;
    /*<         XS(I) = 3.0*((X(I+1)-X(I))*DSM/DSP + (X(I)-X(I-1))*DSP/DSM) >*/
    xs[i__] = ((x[i__ + 1] - x[i__]) * dsm / dsp + (x[i__] - x[i__ - 1]) *
        dsp / dsm) * 3.f;
  }

  /* ---- set zero 3rd derivative end conditions */
  a[0] = 1.f;
  c__[0] = 1.f;
  xs[1] = (x[2] - x[1]) * 2.f / (s[2] - s[1]);
  b[*n - 1] = 1.f;
  a[*n - 1] = 1.f;
  xs[*n] = (x[*n] - x[*n - 1]) * 2.f / (s[*n] - s[*n - 1]);
  
  if (*n == 2) {
    /* ----- if only two points are present, specify zero 2nd derivative instead */
    /* -     (straight line interpolation will result) */
    b[*n - 1] = 1.f;
    a[*n - 1] = 2.f;
    xs[*n] = (x[*n] - x[*n - 1]) * 3.f / (s[*n] - s[*n - 1]);
  }
  /* ---- solve for derivative array XS */
  trisol_(a, b, c__, &xs[1], n);
  return 0;
} /* spline_ */

int splind_(real *x, real *xs, real *s, int *n, real *xs1, real *xs2)
{
  /* System generated locals */
  int i__1;

  /* Local variables */
  static real a[5001], b[5001], c__[5001];
  static int i__;
  static real dsm, dsp;

  /* ------------------------------------------------------- */
  /*     Calculates spline coefficients for X(S).          | */
  /*     Same as SPLINE, but also allows specified-slope   | */
  /*     or zero-curvature end conditions to be imposed.   | */
  /*                                                       | */
  /*     To evaluate the spline at some value of S,        | */
  /*     use SEVAL and/or DEVAL.                           | */
  /*                                                       | */
  /*     S        independent variable array (input)       | */
  /*     X        dependent variable array   (input)       | */
  /*     XS       dX/dS array                (calculated)  | */
  /*     N        number of points           (input)       | */
  /*     XS1,XS2  endpoint derivatives       (input)       | */
  /*              If =  999.0, then usual zero second      | */
  /*              derivative end condition(s) are used     | */
  /*              If = -999.0, then zero third             | */
  /*              derivative end condition(s) are used     | */
  /*                                                       | */
  /*     Note: specifying both XS1,XS2 = -999.0            | */
  /*           is equivalent to using SPLINE.              | */
  /*                                                       | */
  /* ------------------------------------------------------- */
  /*<       IF(N.GT.NMAX) STOP 'SPLIND: array overflow, increase NMAX' >*/
  /* Parameter adjustments */
  --s;
  --xs;
  --x;

  /* Function Body */
  if (*n > 5001) {
    s_stop("SPLIND: array overflow, increase NMAX", (ftnlen)37);
  }

  /*<       DO 1 I=2, N-1 >*/
  i__1 = *n - 1;
  for (i__ = 2; i__ <= i__1; ++i__) {
    /*<         DSM = S(I) - S(I-1) >*/
    dsm = s[i__] - s[i__ - 1];
    /*<         DSP = S(I+1) - S(I) >*/
    dsp = s[i__ + 1] - s[i__];
    /*<         B(I) = DSP >*/
    b[i__ - 1] = dsp;
    /*<         A(I) = 2.0*(DSM+DSP) >*/
    a[i__ - 1] = (dsm + dsp) * 2.f;
    /*<         C(I) = DSM >*/
    c__[i__ - 1] = dsm;
    /*<         XS(I) = 3.0*((X(I+1)-X(I))*DSM/DSP + (X(I)-X(I-1))*DSP/DSM) >*/
    xs[i__] = ((x[i__ + 1] - x[i__]) * dsm / dsp + (x[i__] - x[i__ - 1]) *
        dsp / dsm) * 3.f;
  }

  if (*xs1 == 999.f) {
    /* ----- set zero second derivative end condition */
    /*<        A(1) = 2.0 >*/
    a[0] = 2.f;
    /*<        C(1) = 1.0 >*/
    c__[0] = 1.f;
    /*<        XS(1) = 3.0*(X(2)-X(1)) / (S(2)-S(1)) >*/
    xs[1] = (x[2] - x[1]) * 3.f / (s[2] - s[1]);
  } 
  else if (*xs1 == -999.f) 
  {
    /* ----- set zero third derivative end condition */
    /*<        A(1) = 1.0 >*/
    a[0] = 1.f;
    /*<        C(1) = 1.0 >*/
    c__[0] = 1.f;
    /*<        XS(1) = 2.0*(X(2)-X(1)) / (S(2)-S(1)) >*/
    xs[1] = (x[2] - x[1]) * 2.f / (s[2] - s[1]);
  } 
  else 
  {
    /* ----- set specified first derivative end condition */
    a[0] = 1.f;
    c__[0] = 0.f;
    xs[1] = *xs1;
  }

  if (*xs2 == 999.f) 
  {
    b[*n - 1] = 1.f;
    a[*n - 1] = 2.f;
    xs[*n] = (x[*n] - x[*n - 1]) * 3.f / (s[*n] - s[*n - 1]);
  } 
  else if (*xs2 == -999.f) 
  {
    b[*n - 1] = 1.f;
    a[*n - 1] = 1.f;
    xs[*n] = (x[*n] - x[*n - 1]) * 2.f / (s[*n] - s[*n - 1]);
  } 
  else 
  {
    a[*n - 1] = 1.f;
    b[*n - 1] = 0.f;
    xs[*n] = *xs2;
  }

  /*<       IF(N.EQ.2 .AND. XS1.EQ.-999.0 .AND. XS2.EQ.-999.0) THEN >*/
  if (*n == 2 && *xs1 == -999.f && *xs2 == -999.f) 
  {
    b[*n - 1] = 1.f;
    a[*n - 1] = 2.f;
    /*<        XS(N) = 3.0*(X(N)-X(N-1)) / (S(N)-S(N-1)) >*/
    xs[*n] = (x[*n] - x[*n - 1]) * 3.f / (s[*n] - s[*n - 1]);
  }

  /* ---- solve for derivative array XS */
  trisol_(a, b, c__, &xs[1], n);
  return 0;
} /* splind_ */

int splina_(real *x, real *xs, real *s, int *n)
{
  /* System generated locals */
  int i__1;

  /* Local variables */
  static int i__;
  static real ds, dx, xs1, xs2;
  static logical lend;

  /* ------------------------------------------------------- */
  /*     Calculates spline coefficients for X(S) by a      | */
  /*     simple averaging of adjacent segment slopes.      | */
  /*                                                       | */
  /*     Interpolated X(S) is less likely to oscillate     | */
  /*     than with SPLINE, but does not have continuity    | */
  /*     in curvature.                                     | */
  /*                                                       | */
  /*     To evaluate the spline at some value of S,        | */
  /*     use SEVAL and/or DEVAL.                           | */
  /*                                                       | */
  /*     S        independent variable array (input)       | */
  /*     X        dependent variable array   (input)       | */
  /*     XS       dX/dS array                (calculated)  | */
  /*     N        number of points           (input)       | */
  /*                                                       | */
  /* ------------------------------------------------------- */
  /* Parameter adjustments */
  --s;
  --xs;
  --x;

  /* Function Body */
  lend = TRUE_;
    
  i__1 = *n - 1;
  for (i__ = 1; i__ <= i__1; ++i__) 
  {
    ds = s[i__ + 1] - s[i__];
    if (ds == 0.f) {
      xs[i__] = xs1;
      lend = TRUE_;
    } 
    else 
    {
      dx = x[i__ + 1] - x[i__];
      xs2 = dx / ds;
      if (lend) {
        xs[i__] = xs2;
        lend = FALSE_;
      } 
      else 
      {
        xs[i__] = (xs1 + xs2) * .5f;
      }
    }
    xs1 = xs2;
  }
  /*<       XS(N) = XS1 >*/
  xs[*n] = xs1;
  return 0;
} /* splina_ */

int trisol_(real *a, real *b, real *c__, real *d__, int * kk)
{
  /* System generated locals */
  int i__1;

  /* Local variables */
  static int k, km;

  /*<       DIMENSION A(KK),B(KK),C(KK),D(KK) >*/
  /* ----------------------------------------- */
  /*     Solves KK long, tri-diagonal system | */
  /*                                         | */
  /*             A C          D              | */
  /*             B A C        D              | */
  /*               B A .      .              | */
  /*                 . . C    .              | */
  /*                   B A    D              | */
  /*                                         | */
  /*     The righthand side D is replaced by | */
  /*     the solution.  A, C are destroyed.  | */
  /* ----------------------------------------- */

  /* Parameter adjustments */
  --d__;
  --c__;
  --b;
  --a;

  /* Function Body */
  i__1 = *kk;
  for (k = 2; k <= i__1; ++k) 
  {
    km = k - 1;
    c__[km] /= a[km];
    d__[km] /= a[km];
    a[k] -= b[k] * c__[km];
    d__[k] -= b[k] * d__[km];
  }
  d__[*kk] /= a[*kk];
  for (k = *kk - 1; k >= 1; --k) 
  {
    d__[k] -= c__[k] * d__[k + 1];
  }
  return 0;
} /* trisol_ */

doublereal geval_(real *ss, real *x, real *xs, real *s, int *n)
{
  /* System generated locals */
  int i__1;
  real ret_val;

  /* Local variables */
  static int i__, k;
  static real t, ds, cx1, cx2;
  static int imid;
  static real dgev;
  static int ilow;

  /*<       DIMENSION X(N),XS(N),S(N) >*/
  /* -------------------------------------------------- */
  /*     Calculates int( X(SS) ) dS                   | */
  /*     XS array must have been calculated by SPLINE | */
  /* -------------------------------------------------- */
  /*<       ILOW = 1 >*/
  /* Parameter adjustments */
  --s;
  --xs;
  --x;

  /* Function Body */
  ilow = 1;
  
  i__ = *n;

L10:
  if (i__ - ilow <= 1) 
  {
    goto L11;
  }
  imid = (i__ + ilow) / 2;
  if (*ss < s[imid]) 
  {
    i__ = imid;
  } 
  else 
  {
    ilow = imid;
  }
  goto L10;
L11:

  /* ---- first integrate up to I-1 point */
  /*<       GEVAL = 0. >*/
  ret_val = 0.f;
  i__1 = i__ - 1;
  for (k = 2; k <= i__1; ++k) 
  {
    /*<         DS = S(K) - S(K-1) >*/
    ds = s[k] - s[k - 1];

    /* ------ Int X(t) dt  for t = 0..1 */
    /*<         DGEV = 0.5*(X(K) + X(K-1)) + (XS(K-1) - XS(K))*DS/12.0 >*/
    dgev = (x[k] + x[k - 1]) * .5f + (xs[k - 1] - xs[k]) * ds / 12.f;
    ret_val += dgev * ds;
  }

  /* ---- now integrate up to SS value in I-1..I interval */
  /*<       DS = S(I) - S(I-1) >*/
  ds = s[i__] - s[i__ - 1];
  /*<       T = (SS - S(I-1)) / DS >*/
  t = (*ss - s[i__ - 1]) / ds;
  /*<       CX1 = DS*XS(I-1) - X(I) + X(I-1) >*/
  cx1 = ds * xs[i__ - 1] - x[i__] + x[i__ - 1];
  /*<       CX2 = DS*XS(I)   - X(I) + X(I-1) >*/
  cx2 = ds * xs[i__] - x[i__] + x[i__ - 1];

  dgev = t * .5f * t * x[i__] + (t - t * .5f * t) * x[i__ - 1] + (6.f - t * 
      8.f + t * 3.f * t) * t * t * cx1 / 12.f + (t * -4.f + t * 3.f * t)
      * t * t * cx2 / 12.f;

  ret_val += dgev * ds;

  return ret_val;
} /* geval_ */

/*<       FUNCTION SEVAL(SS,X,XS,S,N) >*/
doublereal seval_(real *ss, real *x, real *xs, real *s, int *n)
{
  /* System generated locals */
  real ret_val;

  /* Local variables */
  static int i__;
  static real t, ds, cx1, cx2;
  static int imid, ilow;

  /* -------------------------------------------------- */
  /*     Calculates X(SS)                             | */
  /*     XS array must have been calculated by SPLINE | */
  /* -------------------------------------------------- */
  /*<       ILOW = 1 >*/
  /* Parameter adjustments */
  --s;
  --xs;
  --x;

  /* Function Body */
  ilow = 1;
  
  i__ = *n;

L10:
  if (i__ - ilow <= 1) 
  {
    goto L11;
  }

  imid = (i__ + ilow) / 2;
  if (*ss < s[imid]) 
  {
    i__ = imid;
  } else {
	ilow = imid;
  }
  /*<       GO TO 10 >*/
  goto L10;

L11:
  ds = s[i__] - s[i__ - 1];
  t = (*ss - s[i__ - 1]) / ds;
  cx1 = ds * xs[i__ - 1] - x[i__] + x[i__ - 1];
  cx2 = ds * xs[i__] - x[i__] + x[i__ - 1];
  ret_val = t * x[i__] + (1.f - t) * x[i__ - 1] + (t - t * t) * ((1.f - t) *
      cx1 - t * cx2);
  return ret_val;
} /* seval_ */

doublereal deval_(real *ss, real *x, real *xs, real *s, int *n)
{
  /* System generated locals */
  real ret_val;

  /* Local variables */
  static int i__;
  static real t, ds, cx1, cx2;
  static int imid, ilow;

  /*<       DIMENSION X(N),XS(N),S(N) >*/
  /* -------------------------------------------------- */
  /*     Calculates dX/dS(SS)                         | */
  /*     XS array must have been calculated by SPLINE | */
  /* -------------------------------------------------- */
  /* Parameter adjustments */
  --s;
  --xs;
  --x;

  /* Function Body */
  ilow = 1;
  i__ = *n;

L10:
  if (i__ - ilow <= 1) 
  {
    goto L11;
  }

  imid = (i__ + ilow) / 2;
  if (*ss < s[imid]) 
  {
    i__ = imid;
  } else {
    ilow = imid;
  }
  goto L10;

L11:
  ds = s[i__] - s[i__ - 1];
  t = (*ss - s[i__ - 1]) / ds;
  cx1 = ds * xs[i__ - 1] - x[i__] + x[i__ - 1];
  cx2 = ds * xs[i__] - x[i__] + x[i__ - 1];
  ret_val = x[i__] - x[i__ - 1] + (1.f - t * 4.f + t * 3.f * t) * cx1 + t * 
      (t * 3.f - 2.f) * cx2;
  ret_val /= ds;
  return ret_val;
} /* deval_ */

doublereal d2val_(real *ss, real *x, real *xs, real *s, int *n)
{
  /* System generated locals */
  real ret_val, r__1;

  /* Local variables */
  static int i__;
  static real t, ds, cx1, cx2;
  static int imid, ilow;

  /* -------------------------------------------------- */
  /*     Calculates d2X/dS2(SS)                       | */
  /*     XS array must have been calculated by SPLINE | */
  /* -------------------------------------------------- */
  /* Parameter adjustments */
  --s;
  --xs;
  --x;

  /* Function Body */
  ilow = 1;
  i__ = *n;
L10:
  if (i__ - ilow <= 1) 
  {
    goto L11;
  }
  imid = (i__ + ilow) / 2;
  if (*ss < s[imid]) 
  {
    i__ = imid;
  } else {
    ilow = imid;
  }
  goto L10;
L11:
  ds = s[i__] - s[i__ - 1];
  t = (*ss - s[i__ - 1]) / ds;
  cx1 = ds * xs[i__ - 1] - x[i__] + x[i__ - 1];
  cx2 = ds * xs[i__] - x[i__] + x[i__ - 1];
  ret_val = (t * 6.f - 4.f) * cx1 + (t * 6.f - 2.f) * cx2;
  /* Computing 2nd power */
  r__1 = ds;
  ret_val /= r__1 * r__1;
  return ret_val;
} /* d2val_ */

int sevall_(real *ss, real *x, real *xs, real *s, int *n,
	 real *xx, real *xxs, real *xxss)
{
    /* System generated locals */
    real r__1;

    /* Local variables */
    static int i__;
    static real t, f0, f1, f2, f3, ds;
    static int imid, ilow;

/* -------------------------------------------------- */
/*     Calculates all spline derivatives.           | */
/*     (Combines SEVAL, DEVAL, D2VAL)               | */
/*     XS array must have been calculated by SPLINE | */
/* -------------------------------------------------- */
  /* Parameter adjustments */
  --s;
  --xs;
  --x;

  /* Function Body */
  ilow = 1;
    i__ = *n;
L10:
  if (i__ - ilow <= 1) 
  {
    goto L11;
  }

  imid = (i__ + ilow) / 2;
  if (*ss < s[imid]) 
  {
    i__ = imid;
  } else {
    ilow = imid;
  }
  goto L10;
L11:
  ds = s[i__] - s[i__ - 1];
  t = (*ss - s[i__ - 1]) / ds;
  
  f0 = x[i__ - 1];
  f1 = ds * xs[i__ - 1];
  f2 = -ds * (xs[i__ - 1] * 2.f + xs[i__]) + (x[i__] - x[i__ - 1]) * 3.f;
  f3 = ds * (xs[i__ - 1] + xs[i__]) - (x[i__] - x[i__ - 1]) * 2.f;

  *xx = f0 + t * (f1 + t * (f2 + t * f3));
  *xxs = f1 + t * (f2 * 2.f + t * 3.f * f3);
  *xxss = f2 * 2.f + t * 6.f * f3;

  *xxs /= ds;
  /* Computing 2nd power */
  r__1 = ds;
  *xxss /= r__1 * r__1;

  return 0;
} /* sevall_ */

int sevlin_(real *ss, real *x, real *s, int *n, real *xx,
	 real *xxs)
{
  static int i__;
  static real t, ds;
  static int imid, ilow;

/* ------------------------------------------------------------ */
/*     Calculates X(SS) and dX/ds(SS) using piecewise-linear  | */
/*     interpolation. This is intended for intepolating very  | */
/*     noisy data for which a cubic spline is inappropriate.  | */
/* ------------------------------------------------------------ */
  /* Parameter adjustments */
  --s;
  --x;

  /* Function Body */
  ilow = 1;
  i__ = *n;
L10:
  if (i__ - ilow <= 1) 
  {
    goto L11;
  }

  imid = (i__ + ilow) / 2;
  if (*ss < s[imid]) {
    i__ = imid;
  } else {
    ilow = imid;
  }
  goto L10;

L11:
  ds = s[i__] - s[i__ - 1];
  t = (*ss - s[i__ - 1]) / ds;
  *xx = t * x[i__] + (1.f - t) * x[i__ - 1];
  *xxs = (x[i__] - x[i__ - 1]) / ds;
  return 0;
} /* sevlin_ */

doublereal curv_(real *ss, real *x, real *xs, real *y, real *ys, real *s, 
	int *n)
{
  /* System generated locals */
  real ret_val, r__1;
  
  /* Builtin functions */
  double sqrt(doublereal);
  
  /* Local variables */
  static int i__;
  static real t, f1, f2, f3, g1, g2, g3, ds, xd, yd, xdd, ydd;
  static int imid, ilow;

/* ----------------------------------------------- */
/*     Calculates curvature of splined 2-D curve | */
/*     at S = SS                                 | */
/*                                               | */
/*     S        arc length array of curve        | */
/*     X, Y     coordinate arrays of curve       | */
/*     XS,YS    derivative arrays                | */
/*              (calculated earlier by SPLINE)   | */
/* ----------------------------------------------- */
  /* Parameter adjustments */
  --s;
  --ys;
  --y;
  --xs;
  --x;

  /* Function Body */
  ilow = 1;
  i__ = *n;
L10:
  if (i__ - ilow <= 1) 
  {
	   goto L11;
  }

  imid = (i__ + ilow) / 2;
  if (*ss < s[imid]) 
  {
    i__ = imid;
  } 
  else 
  {
    ilow = imid;
  }
  goto L10;

L11:
  ds = s[i__] - s[i__ - 1];
  t = (*ss - s[i__ - 1]) / ds;
  f1 = ds * xs[i__ - 1];
  f2 = -ds * (xs[i__ - 1] * 2.f + xs[i__]) + (x[i__] - x[i__ - 1]) * 3.f;
  f3 = ds * (xs[i__ - 1] + xs[i__]) - (x[i__] - x[i__ - 1]) * 2.f;

  xd = f1 + t * (f2 * 2.f + t * 3.f * f3);
  xdd = f2 * 2.f + t * 6.f * f3;

  g1 = ds * ys[i__ - 1];
  g2 = -ds * (ys[i__ - 1] * 2.f + ys[i__]) + (y[i__] - y[i__ - 1]) * 3.f;
  g3 = ds * (ys[i__ - 1] + ys[i__]) - (y[i__] - y[i__ - 1]) * 2.f;

  yd = g1 + t * (g2 * 2.f + t * 3.f * g3);
  ydd = g2 * 2.f + t * 6.f * g3;

  /* Computing 3rd power */
  r__1 = xd * xd + yd * yd;
  ret_val = (xd * ydd - yd * xdd) / sqrt(r__1 * (r__1 * r__1));
  return ret_val;
} /* curv_ */

/*<       FUNCTION CURVS(SS,X,XS,Y,YS,S,N) >*/
doublereal curvs_(real *ss, real *x, real *xs, real *y, real *ys, real *s, 
	int *n)
{
  /* System generated locals */
  real ret_val, r__1;

  /* Builtin functions */
  double sqrt(doublereal);

  /* Local variables */
  static int i__;
  static real t, f1, f2, f3, g1, g2, g3, ds, xd, yd, cx1, cx2, cy1, cy2, 
    xdd, ydd, bot, top;
  static int imid;
  static real xddd, yddd;
  static int ilow;
  static real sqrtb, dbotdt, dtopdt;

/*<       DIMENSION X(N), XS(N), Y(N), YS(N), S(N) >*/
/* ----------------------------------------------- */
/*     Calculates curvature derivative of        | */
/*     splined 2-D curve at S = SS               | */
/*                                               | */
/*     S        arc length array of curve        | */
/*     X, Y     coordinate arrays of curve       | */
/*     XS,YS    derivative arrays                | */
/*              (calculated earlier by SPLINE)   | */
/* ----------------------------------------------- */

  /* Parameter adjustments */
  --s;
  --ys;
  --y;
  --xs;
  --x;

  /* Function Body */
  ilow = 1;
  i__ = *n;

L10:
  if (i__ - ilow <= 1) 
  {
    goto L11;
  }

  imid = (i__ + ilow) / 2;
  if (*ss < s[imid]) {
    i__ = imid;
  } 
  else 
  {
    ilow = imid;
  }
  goto L10;

L11:
  ds = s[i__] - s[i__ - 1];
  t = (*ss - s[i__ - 1]) / ds;

  cx1 = ds * xs[i__ - 1] - x[i__] + x[i__ - 1];
  cx2 = ds * xs[i__] - x[i__] + x[i__ - 1];
  xd = x[i__] - x[i__ - 1] + (1.f - t * 4.f + t * 3.f * t) * cx1 + t * (t * 
      3.f - 2.f) * cx2;
  xdd = (t * 6.f - 4.f) * cx1 + (t * 6.f - 2.f) * cx2;
  xddd = cx1 * 6.f + cx2 * 6.f;

  cy1 = ds * ys[i__ - 1] - y[i__] + y[i__ - 1];
  cy2 = ds * ys[i__] - y[i__] + y[i__ - 1];
  yd = y[i__] - y[i__ - 1] + (1.f - t * 4.f + t * 3.f * t) * cy1 + t * (t * 
      3.f - 2.f) * cy2;  
  ydd = (t * 6.f - 4.f) * cy1 + (t * 6.f - 2.f) * cy2;
  yddd = cy1 * 6.f + cy2 * 6.f;

  f1 = ds * xs[i__ - 1];
  f2 = -ds * (xs[i__ - 1] * 2.f + xs[i__]) + (x[i__] - x[i__ - 1]) * 3.f;
  f3 = ds * (xs[i__ - 1] + xs[i__]) - (x[i__] - x[i__ - 1]) * 2.f;

  xd = f1 + t * (f2 * 2.f + t * 3.f * f3);
  xdd = f2 * 2.f + t * 6.f * f3;
  xddd = f3 * 6.f;

  g1 = ds * ys[i__ - 1];
  g2 = -ds * (ys[i__ - 1] * 2.f + ys[i__]) + (y[i__] - y[i__ - 1]) * 3.f;
  g3 = ds * (ys[i__ - 1] + ys[i__]) - (y[i__] - y[i__ - 1]) * 2.f;

  yd = g1 + t * (g2 * 2.f + t * 3.f * g3);
  ydd = g2 * 2.f + t * 6.f * g3;
  yddd = g3 * 6.f;

  sqrtb = sqrt(xd * xd + yd * yd);
  
  /* Computing 3rd power */
  r__1 = sqrtb;
  bot = r__1 * (r__1 * r__1);
  dbotdt = sqrtb * 3.f * (xd * xdd + yd * ydd);

  top = xd * ydd - yd * xdd;
  dtopdt = xd * yddd - yd * xddd;

  /* Computing 2nd power */
  r__1 = bot;
  ret_val = (dtopdt * bot - dbotdt * top) / (r__1 * r__1) / ds;

  return ret_val;
} /* curvs_ */

int sinvrt_(real *si, real *xi, real *x, real *xs, real *s, 
	int *n)
{
  /* System generated locals */
  real r__1;

  /* Builtin functions */
  int s_wsle(cilist *), do_lio(int *, int *, char *, ftnlen), 
    e_wsle(void);

  /* Local variables */
  static real ds, xx, xxs;
  static int iter;
  static real xxss;
  extern /* Subroutine */ int sevall_(real *, real *, real *, real *, 
    int *, real *, real *, real *);

  /* Fortran I/O blocks */
  static cilist io___111 = { 0, 6, 0, 0, 0 };

/* ---------------------------------------------------- */
/*     Calculates the "inverse" spline function S(X). | */
/*     Since S(X) can be multi-valued or not defined, | */
/*      this is not a "black-box" routine.  The call- | */
/*      ing program must pass via SI a sufficiently   | */
/*      good initial guess for S(XI).                 | */
/*                                                    | */
/*     XI      specified X value       (input)        | */
/*     SI      calculated S(XI) value  (input,output) | */
/*     X,XS,S  usual spline arrays     (input)        | */
/*                                                    | */
/* ---------------------------------------------------- */

  /* Parameter adjustments */
  --s;
  --xs;
  --x;

  /* Function Body */
  for (iter = 1; iter <= 10; ++iter) 
  {
    sevall_(si, &x[1], &xs[1], &s[1], n, &xx, &xxs, &xxss);
    ds = (*xi - xx) / xxs;
    *si += ds;
    if ((r__1 = ds / (s[*n] - s[1]), dabs(r__1)) < 1e-5f) {
      return 0;
    }
  }
  s_wsle(&io___111);
  do_lio(&c__9, &c__1, "SINVRT: spline inversion failed.  Continuing...", (
    ftnlen)47);
  e_wsle();
  return 0;
} /* sinvrt_ */

int scalc_(real *x, real *y, real *s, int *n)
{
  /* System generated locals */
  int i__1;
  real r__1, r__2;

  /* Builtin functions */
  double sqrt(doublereal);

  /* Local variables */
  static int i__;

/*<       DIMENSION X(N),Y(N),S(N) >*/
/* ---------------------------------------- */
/*     Calculates the arc length array S  | */
/*     for a 2-D array of points (X,Y).   | */
/* ---------------------------------------- */

  /* Parameter adjustments */
  --s;
  --y;
  --x;

  /* Function Body */
  s[1] = 0.f;
  i__1 = *n;
  for (i__ = 2; i__ <= i__1; ++i__) {
    /* Computing 2nd power */
    r__1 = x[i__] - x[i__ - 1];
    r__2 = y[i__] - y[i__ - 1];
    s[i__] = s[i__ - 1] + sqrt(r__1 * r__1 + r__2 * r__2);
  }

  return 0;
} /* scalc_ */

int segspl_(real *x, real *xs, real *s, int *n)
{
  /* System generated locals */
  int i__1;

  /* Builtin functions */
  /* Subroutine */ int s_stop(char *, ftnlen);

  /* Local variables */
  static int iseg, nseg, iseg0;

/*<       DIMENSION X(N),XS(N),S(N) >*/
/* ----------------------------------------------- */
/*     Splines X(S) array just like SPLINE,      | */
/*     but allows derivative discontinuities     | */
/*     at segment joints.  Segment joints are    | */
/*     defined by identical successive S values. | */
/* ----------------------------------------------- */

  --s;
  --xs;
  --x;

  /* Function Body */
  if (s[1] == s[2]) 
  {
  	s_stop("SEGSPL:  First input point duplicated", (ftnlen)37);
  }
  if (s[*n] == s[*n - 1]) {
    s_stop("SEGSPL:  Last  input point duplicated", (ftnlen)37);
  }

  iseg0 = 1;
  i__1 = *n - 2;
  for (iseg = 2; iseg <= i__1; ++iseg) {
    if (s[iseg] == s[iseg + 1]) {
      nseg = iseg - iseg0 + 1;
      spline_(&x[iseg0], &xs[iseg0], &s[iseg0], &nseg);
      iseg0 = iseg + 1;
    }
  }

  nseg = *n - iseg0 + 1;
  spline_(&x[iseg0], &xs[iseg0], &s[iseg0], &nseg);

  return 0;
} /* segspl_ */

int segspd_(real *x, real *xs, real *s, int *n, real *
	xs1, real *xs2)
{
  /* System generated locals */
  int i__1;

  /* Local variables */
  static int iseg, nseg, iseg0;

/*<       DIMENSION X(N),XS(N),S(N) >*/
/* ----------------------------------------------- */
/*     Splines X(S) array just like SPLIND,      | */
/*     but allows derivative discontinuities     | */
/*     at segment joints.  Segment joints are    | */
/*     defined by identical successive S values. | */
/* ----------------------------------------------- */

  /* Parameter adjustments */
  --s;
  --xs;
  --x;

  /* Function Body */
  if (s[1] == s[2]) {
  	s_stop("SEGSPD:  First input point duplicated", (ftnlen)37);
  }
  if (s[*n] == s[*n - 1]) {
    s_stop("SEGSPD:  Last  input point duplicated", (ftnlen)37);
  }

  iseg0 = 1;
  i__1 = *n - 2;
  for (iseg = 2; iseg <= i__1; ++iseg) 
  {
    if (s[iseg] == s[iseg + 1]) {
      nseg = iseg - iseg0 + 1;
      splind_(&x[iseg0], &xs[iseg0], &s[iseg0], &nseg, xs1, xs2);
      iseg0 = iseg + 1;
    }
  }

  nseg = *n - iseg0 + 1;
  splind_(&x[iseg0], &xs[iseg0], &s[iseg0], &nseg, xs1, xs2);

  return 0;
} /* segspd_ */

/*<    >*/
/* Subroutine */ int inters_(logical *ok, real *ss1, real *ss2, real *x1, 
	real *xs1, real *y1, real *ys1, real *s1, int *n1, real *x2, real 
	*xs2, real *y2, real *ys2, real *s2, int *n2)
{
  /* Initialized data */

  static real eps = 1e-5f;

  /* System generated locals */
  real r__1, r__2;

  /* Builtin functions */
  int s_wsle(cilist *), do_lio(int *, int *, char *, ftnlen), 
    e_wsle(void);

  /* Local variables */
  static real a11, a12, a21, a22, ds1, ds2, rs1, rs2, xx1, xx2, yy1, yy2, 
    det, rlx;
  static int iter, irlx;
  static logical clip1, clip2;
  extern doublereal deval_(real *, real *, real *, real *, int *), 
    seval_(real *, real *, real *, real *, int *);
  static real rs1old, ss1old, ss2old, rs2old;

  /* Fortran I/O blocks */
  static cilist io___137 = { 0, 6, 0, 0, 0 };
  static cilist io___143 = { 0, 6, 0, 0, 0 };
  static cilist io___144 = { 0, 6, 0, 0, 0 };
  static cilist io___145 = { 0, 6, 0, 0, 0 };


/*<       LOGICAL OK >*/
/*<       DIMENSION X1(N1),XS1(N1),Y1(N1),YS1(N1),S1(N1) >*/
/*<       DIMENSION X2(N2),XS2(N2),Y2(N2),YS2(N2),S2(N2) >*/
/* ------------------------------------------------------- */
/*     Finds spline coordinate values SS1, SS2 at the */
/*     intersection of two space curves (X1,Y1), (X2,Y2). */
/* ------------------------------------------------------- */
  /* Parameter adjustments */
  --s1;
  --ys1;
  --y1;
  --xs1;
  --x1;
  --s2;
  --ys2;
  --y2;
  --xs2;
  --x2;

  /* Function Body */

  *ok = TRUE_;
  rs1 = 1e12f;
  rs2 = 1e12f;
  ds1 = 0.f;
  ds2 = 0.f;

  for (iter = 1; iter <= 12; ++iter) 
  {
    rlx = 1.f;
    ss1old = *ss1;
    ss2old = *ss2;
    rs1old = dabs(rs1);
    rs2old = dabs(rs2);

    /*<         DO 10 IRLX=1, 16 >*/
    for (irlx = 1; irlx <= 16; ++irlx) {

      clip1 = FALSE_;
      clip2 = FALSE_;
      *ss1 = ss1old + rlx * ds1;
      *ss2 = ss2old + rlx * ds2;

      if (*ss1 < s1[1] || *ss1 > s1[*n1]) 
      {
        clip1 = TRUE_;
        *ss1 = dmax(*ss1,s1[1]);
        /* Computing MIN */
        r__1 = *ss1, r__2 = s1[*n1];
        *ss1 = dmin(r__1,r__2);
      }
      if (*ss2 < s2[1] || *ss2 > s2[*n2]) {
        clip2 = TRUE_;
        *ss2 = dmax(*ss2,s2[1]);
        /* Computing MIN */
        r__1 = *ss2, r__2 = s2[*n2];
        *ss2 = dmin(r__1,r__2);
      }

      xx1 = seval_(ss1, &x1[1], &xs1[1], &s1[1], n1);
      xx2 = seval_(ss2, &x2[1], &xs2[1], &s2[1], n2);
      yy1 = seval_(ss1, &y1[1], &ys1[1], &s1[1], n1);
      yy2 = seval_(ss2, &y2[1], &ys2[1], &s2[1], n2);

      rs1 = xx1 - xx2;
      rs2 = yy1 - yy2;

      if (dabs(rs1) < rs1old && dabs(rs2) < rs2old) {
        goto L11;
      }

      rlx *= .5f;
    }
    s_wsle(&io___137);
    do_lio(&c__9, &c__1, "INTERS: Under-relaxation loop failed.", (ftnlen)
          37);
    e_wsle();
L11:

  a11 = deval_(ss1, &x1[1], &xs1[1], &s1[1], n1);
  a12 = -deval_(ss2, &x2[1], &xs2[1], &s2[1], n2);
  a21 = deval_(ss1, &y1[1], &ys1[1], &s1[1], n1);
  a22 = -deval_(ss2, &y2[1], &ys2[1], &s2[1], n2);

  det = a11 * a22 - a12 * a21;
  ds1 = -(rs1 * a22 - a12 * rs2) / det;
  ds2 = -(a11 * rs2 - rs1 * a21) / det;

  if (dabs(ds1) < eps * (s1[*n1] - s1[1]) && dabs(ds2) < eps * (s2[*n2] 
      - s2[1])) {
    return 0;
      }
  }
  
  s_wsle(&io___143);
  do_lio(&c__9, &c__1, "INTERS: Convergence failed. Res =", (ftnlen)33);
  do_lio(&c__4, &c__1, (char *)&rs1, (ftnlen)sizeof(real));
  do_lio(&c__4, &c__1, (char *)&rs2, (ftnlen)sizeof(real));
  e_wsle();
  
  if (clip1) {
    s_wsle(&io___144);
    do_lio(&c__9, &c__1, "        S1 clip:", (ftnlen)16);
    do_lio(&c__4, &c__1, (char *)&s1[1], (ftnlen)sizeof(real));
    do_lio(&c__4, &c__1, (char *)&s1[*n1], (ftnlen)sizeof(real));
    do_lio(&c__4, &c__1, (char *)&(*ss1), (ftnlen)sizeof(real));
    do_lio(&c__4, &c__1, (char *)&ds1, (ftnlen)sizeof(real));
    e_wsle();
  }
  
  if (clip2) {
    s_wsle(&io___145);
    do_lio(&c__9, &c__1, "        S2 clip:", (ftnlen)16);
    do_lio(&c__4, &c__1, (char *)&s2[1], (ftnlen)sizeof(real));
    do_lio(&c__4, &c__1, (char *)&s2[*n2], (ftnlen)sizeof(real));
    do_lio(&c__4, &c__1, (char *)&(*ss2), (ftnlen)sizeof(real));
    do_lio(&c__4, &c__1, (char *)&ds2, (ftnlen)sizeof(real));
    e_wsle();
  }
  
  *ok = FALSE_;
  return 0;
} /* inters_ */

int nearpt_(real *xpnt, real *ypnt, real *snear, real *x, 
	real *xp, real *y, real *yp, real *s, int *n)
{

    /* Local variables */
    static real x2i, y2i, dsn, eps, res, xpi, ypi, xxi, yyi, res_s__;
    static int ipass;

    /* Fortran I/O blocks */
    static cilist io___157 = { 0, 6, 0, 0, 0 };

/*<       IMPLICIT REAL (A-H,M,O-Z) >*/
/*<       DIMENSION X(N),XP(N),Y(N),YP(N),S(N) >*/
/* ======================================================== */
/*     Finds arc length position S=SNEAR of a point */
/*     on a 2-D splined curve X(S),Y(S) nearest the */
/*     specified point XPNT,YPNT. */

/*     Assumes the value passed in via SNEAR is a good */
/*     initial guess. */
/* ======================================================== */

/* ---- convergence tolerance */
/*<       EPS = 1.0E-4 * (S(N) - S(1)) >*/
  /* Parameter adjustments */
  --s;
  --yp;
  --y;
  --xp;
  --x;

  /* Function Body */
  eps = (s[*n] - s[1]) * 1e-4f;

  /* ---- Newton iteration loop */
  for (ipass = 1; ipass <= 10; ++ipass) 
  {
    sevall_(snear, &x[1], &xp[1], &s[1], n, &xxi, &xpi, &x2i);
    sevall_(snear, &y[1], &yp[1], &s[1], n, &yyi, &ypi, &y2i);

    /* ------ residual is dot product with curve tangent vector */
    res = (xxi - *xpnt) * xpi + (yyi - *ypnt) * ypi;

    res_s__ = xpi * xpi + ypi * ypi + (xxi - *xpnt) * x2i + (yyi - *ypnt) 
        * y2i;

    dsn = -res / res_s__;
    *snear += dsn;
    if (dabs(dsn) < eps) {
      goto L216;
    }
  }
  s_wsle(&io___157);
  do_lio(&c__9, &c__1, "NEARPT: Convergence failed.  Continuing...", (
          ftnlen)42);
  e_wsle();
L216:

  return 0;
} /* nearpt_ */

